/* ttt-server.c - tic-tac-toe game server
 *
 * Copyright © 2005 Carl Worth
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Carl Worth <cworth@cworth.org>
 */

#include "ttt.h"
#include "ttt-error.h"

#ifndef _TTT_SERVER_H_
#define _TTT_SERVER_H_

/* Register a new client with the server.
 * 
 * Returns: TTT_ERROR_NONE on success, else TTT_ERROR_INVALIDNAME if client's
 * name is not unique.
 *
 * Locking: The server mutex will be acquired and held throughout the
 * execution of this function.
 *
 * Errors: If an error (such as out-of-memory) occurs, this function
 * will not return.
 */
ttt_error_t
ttt_server_register_client (ttt_server_t *server, ttt_client_t *client);

/* Un-register a client from the server.
 *
 * Locking: The server mutex will be acquired and held throughout the
 * execution of this function.
 */
void
ttt_server_unregister_client (ttt_server_t *server, ttt_client_t *client);

/* Send a message to all connected clients.
 *
 * Locking: The server mutex will be acquired and held throughout the
 * execution of this function. Each client mutex may also be acquired
 * and held by functions called during the execution of this function.
 *
 * Errors: If an error such as an IO error occurs, this function will
 * not return.
 */
void
ttt_server_broadcast (ttt_server_t *server, const char *message);


/* Returns the WHO response. The return string is allocated in this
 * function and will need to be free'd by the caller.
 *
 * Locking: The server mutex will be acquired and held throughout the
 * execution of this function. Each client mutex may also be acquired
 * and held by functions called during the execution of this function.
 *
 * Errors: If an error such as an IO error occurs, this function will
 * not return.
 */
const char*
ttt_server_who (ttt_server_t *server);

/* Generates the statistics for the user. If the function does not
 * return an error, the response will be allocated in this function
 * and will need to be free'd by the caller.
 *
 * Locking: The server mutex will be acquired and held throughout the
 * execution of this function. Each client mutex may also be acquired
 * and held by functions called during the execution of this function.
 *
 * Errors: If an error such as an IO error occurs, this function will
 * not return.
 */
ttt_error_t
ttt_server_statistics (ttt_server_t *server,
		       const char *username,
		       char **response);

/* Gets the server hostname.
 *
 */
const char*
ttt_server_get_host (ttt_server_t *server);

/* Gets the server port
 *
 */
const char*
ttt_server_get_port (ttt_server_t *server);

#endif /* _TTT_SERVER_H_ */
